/*
 *
 * File:	getSunTimes.c
 *
 * Function:	This file contains the routine to
 *		calculate sunrise and sunset based
 *		on day of the year, daylight savings,
 *		time zone, latitude, and longitude.
 *
 * Version:	1.0
 *
 * History:	Initial Release
 *
 *
 * Copyright (c) 2016 May 6 by Philip Vrbancic
 *
 * 6223 East Peabody Street
 * Long Beach, California  90808
 * United States of America
 *
 * All Rights Reserved
 *
 * This software is the confidential and proprietary
 * intellectual property of Walland Philip Vrbancic, Jr.
 *
 */


#define EXTERNAL


#include		<stdio.h>
#include		<string.h>
#include		<math.h>

#include		"standardDefs.h"
#include		"parameterDefs.h"
#include		"globalDefs.h"
#include		"protoTypes.h"


#define radianConvert	pi / 180.0
#define degreeConvert	180.0 / pi



VOID
getSunTimes( INT32 days, INT32 dstFlag, FLT32 *riseTimePtr, FLT32 *setTimePtr )

{
  // CHAR			onoff[2][4] = { "Off", "On " };

  INT32			lRiseQuad;
  INT32			lSetQuad;
  INT32			raRiseQuad;
  INT32			raSetQuad;
  // INT32			hour;
  // INT32			min;
  // INT32			sec;
  // INT32			frac;

  // FLT32			zenith;
  FLT32			longitudeHour;
  FLT32			tRise;
  FLT32			tSet;
  FLT32			mRise;
  FLT32			mSet;
  FLT32			lRise;
  FLT32			lSet;
  FLT32			raRise;
  FLT32			raSet;
  FLT32			sinDecRise;
  FLT32			sinDecSet;
  FLT32			cosDecRise;
  FLT32			cosDecSet;
  FLT32			cosHrise;
  FLT32			cosHset;
  FLT32			hrise;
  FLT32			hset;
  FLT32			trise;
  FLT32			tset;
  FLT32			uTrise;
  FLT32			uTset;
  FLT32			localTrise;
  FLT32			localTset;

  FLT32			pi = 3.14159265;


  // zenith = 90.83333333;		// offical (90 degrees, 50 minutes)
  // zenith = 96.0;			// civil
  // zenith = 102.0;			// nautical
  // zenith = 108.0;			// astronomical


// Note assignments to parameters.

  // sprintf( Mesg, "(getSunTimes)           Zenith = %f\n", Zenith );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes)  Local Time Zone = %d\n", TimeZone );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes) Daylight Savings = %s\n\n", onoff[dstFlag] );
  // info( Mesg, SCREEN|LOGFILE );


// Convert the longitude to an hour value and calculate an approximate time.

  longitudeHour = Longitude / 15.0;

  tRise = ( FLT32 )days + (  6.0 - longitudeHour ) / 24.0;
  tSet  = ( FLT32 )days + ( 18.0 - longitudeHour ) / 24.0;

  // sprintf( Mesg, "(getSunTimes) longitudeHour = %f\n\n", longitudeHour );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes) tRise = %f, tSet = %f\n", tRise, tSet );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the mean anomaly of the sun.

  // mRise = tRise * 0.9856 - 3.289;
  // mSet  = tSet  * 0.9856 - 3.289;

  mRise = tRise * 0.98560028 - 3.289;
  mSet  = tSet  * 0.98560028 - 3.289;

  // sprintf( Mesg, "(getSunTimes) mRise = %f, mSet = %f\n\n", mRise, mSet );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the true longitude of the sun and adjust range to (0, 360).

  // lRise = mRise + 1.916 * sin( mRise * radianConvert ) + 0.020 * sin( 2.0 * mRise * radianConvert ) + 102.634 + 180.0; 
  // lSet  = mSet  + 1.916 * sin( mSet  * radianConvert ) + 0.020 * sin( 2.0 * mSet  * radianConvert ) + 102.634 + 180.0;

  lRise = mRise + 1.9148 * sin( mRise * radianConvert ) + 0.020 * sin( 2.0 * mRise * radianConvert ) + 0.0003 * sin( 3.0 * mRise * radianConvert ) + 102.9372 + 180.0; 
  lSet  = mSet  + 1.9148 * sin( mSet  * radianConvert ) + 0.020 * sin( 2.0 * mSet  * radianConvert ) + 0.0003 * sin( 3.0 * mSet  * radianConvert ) + 102.9372 + 180.0;

  // sprintf( Mesg, "(getSunTimes) lRise = %f, lSet = %f\n", lRise, lSet );
  // info( Mesg, SCREEN|LOGFILE );

  lRise = adjustRange( lRise, 360.0 );
  lSet  = adjustRange( lSet , 360.0 );

  // sprintf( Mesg, "(getSunTimes) lRise = %f, lSet = %f (adjusted)\n\n", lRise, lSet );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the right ascension of the sun and adjust range to (0, 360).

  raRise = atan( 0.91764 * tan( lRise * radianConvert ) ) * degreeConvert;
  raSet  = atan( 0.91764 * tan( lSet  * radianConvert ) ) * degreeConvert;

  // sprintf( Mesg, "(getSunTimes) raRise = %f, raSet = %f\n", raRise, raSet );
  // info( Mesg, SCREEN|LOGFILE );

  raRise = adjustRange( raRise, 360.0 );
  raSet  = adjustRange( raSet , 360.0 );

  // sprintf( Mesg, "(getSunTimes) raRise = %f, raSet = %f (adjusted)\n\n", raRise, raSet );
  // info( Mesg, SCREEN|LOGFILE );


// Put the right ascention value into the same quadrant as the true longitude of the sun.

  lRiseQuad  = 90 * floor( lRise / 90.0 );
  lSetQuad   = 90 * floor( lSet  / 90.0 );

  raRiseQuad = 90 * floor( raRise / 90.0 );
  raSetQuad  = 90 * floor( raSet  / 90.0 );

  raRise += lRiseQuad - raRiseQuad;
  raSet  += lSetQuad  - raSetQuad;

  // sprintf( Mesg, "(getSunTimes) lRiseQuad  = %d, lSetQuad  = %d\n", lRiseQuad, lSetQuad );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes) raRiseQuad = %d, raSetQuad = %d\n\n", raRiseQuad, raSetQuad );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes) raRise = %f, raSet = %f (degrees)\n", raRise, raSet );
  // info( Mesg, SCREEN|LOGFILE );


// Convert the right ascention value (degrees) into hours.

  raRise = raRise / 15.0;
  raSet  = raSet  / 15.0;

  // sprintf( Mesg, "(getSunTimes) raRise = %f, raSet = %f (hours)\n\n", raRise, raSet );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the declination of the sun.

  sinDecRise = 0.39782 * sin( lRise * radianConvert );
  sinDecSet  = 0.39782 * sin( lSet  * radianConvert );

  cosDecRise = cos( asin( sinDecRise ) );
  cosDecSet  = cos( asin( sinDecSet  ) );

  // sprintf( Mesg, "(getSunTimes) sinDecRise = %f, sinDecSet = %f\n", sinDecRise, sinDecSet );
  // info( Mesg, SCREEN|LOGFILE );

  // sprintf( Mesg, "(getSunTimes) cosDecRise = %f, cosDecSet = %f\n\n", cosDecRise, cosDecSet );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the local hour angle of the sun.

  cosHrise = ( cos( Zenith * radianConvert ) - ( sinDecRise * sin( Latitude * radianConvert ) ) ) / ( cosDecRise * cos( Latitude * radianConvert ) );
  cosHset  = ( cos( Zenith * radianConvert ) - ( sinDecSet  * sin( Latitude * radianConvert ) ) ) / ( cosDecSet  * cos( Latitude * radianConvert ) );

  // sprintf( Mesg, "(getSunTimes) cosHrise = %f, cosHset = %f\n\n", cosHrise, cosHset );
  // info( Mesg, SCREEN|LOGFILE );

  if( cosHrise > 1 )
  {
    // sprintf( Mesg, "(getSunTimes) The sun never rises on this location at this time.\n\n" );
    // info( Mesg, SCREEN|LOGFILE );
  }

  if( cosHrise < -1 )
  {
    // sprintf( Mesg, "(getSunTimes) The sun never sets on this location at this time.\n\n" );
    // info( Mesg, SCREEN|LOGFILE );
  }

  if( cosHset > 1 )
  {
    // sprintf( Mesg, "(getSunTimes) The sun never rises on this location at this time.\n\n" );
    // info( Mesg, SCREEN|LOGFILE );
  }

  if( cosHset < -1 )
  {
    // sprintf( Mesg, "(getSunTimes) The sun never sets on this location at this time.\n\n" );
    // info( Mesg, SCREEN|LOGFILE );
  }


// Finish calculating H (in degrees) and convert into hours.

  hrise = 360.0 - acos( cosHrise ) * degreeConvert;
  hset  =         acos( cosHset  ) * degreeConvert;

  // sprintf( Mesg, "(getSunTimes) hrise = %f, hset = %f (degrees)\n", hrise, hset );
  // info( Mesg, SCREEN|LOGFILE );

  hrise = hrise / 15.0;
  hset  = hset  / 15.0;

  // sprintf( Mesg, "(getSunTimes) hrise = %f, hset = %f (hours)\n\n", hrise, hset );
  // info( Mesg, SCREEN|LOGFILE );


// Calculate the local mean time of rising/setting.

  trise = hrise + raRise - ( 0.06571 * tRise ) - 6.622;
  tset  = hset  + raSet  - ( 0.06571 * tSet  ) - 6.622;

  // sprintf( Mesg, "(getSunTimes) trise = %f, tset = %f\n\n", trise, tset );
  // info( Mesg, SCREEN|LOGFILE );


// Adjust back to UTC.

  uTrise = trise - longitudeHour;
  uTset  = tset  - longitudeHour;

  // sprintf( Mesg, "(getSunTimes) uTrise = %f, uTset = %f\n", uTrise, uTset );
  // info( Mesg, SCREEN|LOGFILE );


// Adjust time to (0, 23).

  uTrise = adjustRange( uTrise, 24.0 );
  uTset  = adjustRange( uTset , 24.0 );

  // sprintf( Mesg, "(getSunTimes) uTrise = %f, uTset = %f (adjusted)\n\n", uTrise, uTset );
  // info( Mesg, SCREEN|LOGFILE );


// Convert UT value to the local time zone of latitude/longitude.

  localTrise = uTrise + ( FLT32 )TimeZone;
  localTset  = uTset  + ( FLT32 )TimeZone;

  // sprintf( Mesg, "(getSunTimes) localTrise = %f, localTset = %f (time zone)\n", localTrise, localTset );
  // info( Mesg, SCREEN|LOGFILE );


// Account for daylight savings.

  localTrise += ( FLT32 )dstFlag;
  localTset  += ( FLT32 )dstFlag;

  // sprintf( Mesg, "(getSunTimes) localTrise = %f, localTset = %f (with DST)\n", localTrise, localTset );
  // info( Mesg, SCREEN|LOGFILE );


// Adjust time to (0, 23).

  localTrise = adjustRange( localTrise, 24.0 );
  localTset  = adjustRange( localTset , 24.0 );

  // sprintf( Mesg, "(getSunTimes) localTrise = %f, localTset = %f (adjusted)\n\n", localTrise, localTset );
  // info( Mesg, SCREEN|LOGFILE );


// Show time in hours, minutes, and seconds.

  // hour = floor( localTrise );
  // min  = floor( ( localTrise - hour ) * 60.0 );
  // sec  = floor( ( ( localTrise - hour ) * 60.0 - min ) * 60.0 );
  // frac = ( ( ( localTrise - hour ) * 60.0 - min ) * 60.0 - sec ) * 100.0;

  // if( hour > 12 )
  // {
    // sprintf( Mesg, "(getSunTimes) sunrise = %02d:%02d:%02d.%02d PM\n", hour-12, min, sec, frac );
  // }

  // else
  // {
    // sprintf( Mesg, "(getSunTimes) sunrise = %02d:%02d:%02d.%02d AM\n", hour, min, sec, frac );
  // }

  // info( Mesg, SCREEN|LOGFILE );


  // hour = floor( localTset );
  // min  = floor( ( localTset - hour ) * 60.0 );
  // sec  = floor( ( ( localTset - hour ) * 60.0 - min ) * 60.0 );
  // frac = ( ( localTset - hour ) * 60.0 * 60.0 - min * 60.0 - sec ) * 100.0;

  // if( hour > 12 )
  // {
    // sprintf( Mesg, "(getSunTimes) sunset  = %02d:%02d:%02d.%02d PM\n\n", hour-12, min, sec, frac );
  // }

  // else
  // {
    // sprintf( Mesg, "(getSunTimes) sunset  = %02d:%02d:%02d.%02d AM\n\n", hour, min, sec, frac );
  // }

  // info( Mesg, SCREEN|LOGFILE );

  *riseTimePtr = localTrise;
  *setTimePtr  = localTset;
}

