/*
 *
 * File:	lightcontrol.c
 *
 * Function:	This file contains the routine to control
 *		a GPIO at sunrise and at sunset.
 *
 * Version:	1.0
 *
 * History:	Initial Release
 *
 *
 * Copyright (c) 2016 May 6 by Philip Vrbancic
 *
 * 6223 East Peabody Street
 * Long Beach, California  90808
 * United States of America
 *
 * All Rights Reserved
 *
 * This software is the confidential and proprietary
 * intellectual property of Walland Philip Vrbancic, Jr.
 *
 */


#undef EXTERNAL


#include			<stdio.h>
#include			<stdlib.h>
#include			<unistd.h>
#include			<string.h>
#include			<time.h>
#include			<fcntl.h>
#include			<math.h>

#include			<sys/time.h>

#include			"standardDefs.h"
#include			"parameterDefs.h"
#include			"globalDefs.h"
#include			"protoTypes.h"



INT32
main( INT32 argc, CHAR *argv[] )

{
  time_t			timeT;

  STRUCT tm			*tmTime;

  STRUCT timeval		timeVal;
  STRUCT timezone		timeZone;

  CHAR				*timePtr;

  CHAR				testName[CHAR_BUF_LEN];
  CHAR				logPath[PATH_BUF_LEN];

  CHAR				onoff[2][4] = { "Off", "On" };

  INT32				status;
  INT32				sequence;
  INT32				direction;
  INT32				day;
  INT32				month;
  INT32				year;
  INT32				yearDay;
  INT32				dstFlag;
  INT32				days;
  INT32				nextDays;
  INT32				gpioValue;
  INT32				sleepSeconds;

  SINT32			firsTime = TRUE;

  FLT32				localTime;
  FLT32				riseTime;
  FLT32				setTime;
  FLT32				sleepHours;


// Wait for 30 seconds to allow the boot to complete.

  sleep( 30 );


// Get the next log file sequence number.

  sprintf( testName, "%s.", LOG_FILE_LIGHT );

  sequence = genSequence( LIGHTCONTROL_PATH, testName );

  if( sequence == NEGONE )
  {
    exit( EXIT_FAILURE );
  }


// Create the log file and write the current date and time.

  sprintf( logPath, "%s/%s.%02d.log", LIGHTCONTROL_PATH, LOG_FILE_LIGHT, sequence );

  LogFilePtr = fopen( logPath, "w" );

  if( LogFilePtr == ( FILE * )NULL )
  {
    exit( EXIT_FAILURE );
  }

  time( &timeT );

  timePtr = asctime( localtime( &timeT ) );

  sprintf( Mesg, "This Log File is %s.%02d.log, beginning %s\n", LOG_FILE_LIGHT, sequence, timePtr );
  info( Mesg, LOGFILE );


// Get the configuration variables.

  info( "(lightcontrol) Calling readConfig().\n", LOGFILE );

  status = readConfig();

  if( status )
  {
    info( "\n(lightcontrol) Error from readConfig().\n", LOGFILE );

    fclose( LogFilePtr );

    exit( EXIT_FAILURE );
  }


// Show the configuration into the logfile.

  info( "(lightcontrol) Calling showConfig().\n", LOGFILE );

  showConfig( LOGFILE );


// Open the gpio listed in the configuration file.

  sprintf( Mesg, "(lightcontrol) Calling openGPIO() for gpio #%d.\n", Gpio );
  info( Mesg, LOGFILE );

  status = openGPIO( Gpio );

  if( status )
  {
    sprintf( Mesg, "\n(lightcontrol) Error from openGPIO() for gpio #%d.\n\n", Gpio );
    info( Mesg, LOGFILE );

    fclose( LogFilePtr );

    exit( EXIT_FAILURE );
  }


// Set the direction for output.

  direction = 2;

  sprintf( Mesg, "(lightcontrol) Calling ioGPIO() for gpio #%d, direction = %d.\n\n", Gpio, direction );
  info( Mesg, LOGFILE );

  status = ioGPIO( Gpio, direction );

  if( status )
  {
    sprintf( Mesg, "\n(lightcontrol) Error from ioGPIO() for gpio #%d, direction = %d.\n\n", Gpio, direction );
    info( Mesg, LOGFILE );
  }

  info( "(lightcontrol) Entering sunrise/sunset infinite loop.\n\n", LOGFILE );

  fclose( LogFilePtr );


// Enter infinite loop to control gpio at sunrise and at sunset.

  for( ;; )
  {

// Open the log file in append mode and write the current date and time.

    LogFilePtr = fopen( logPath, "a" );

    if( LogFilePtr == ( FILE * )NULL )
    {
      break;
    }

    time( &timeT );

    timePtr = asctime( localtime( &timeT ) );

    sprintf( Mesg, "(lightcontrol) Today is %s", timePtr );
    info( Mesg, LOGFILE );


// Get the present time in hours.

    gettimeofday( &timeVal, &timeZone );

    tmTime = localtime( &timeVal.tv_sec );

    localTime = ( FLT32 )( tmTime->tm_sec + tmTime->tm_min * SEC_MIN + tmTime->tm_hour * SEC_HR ) / ( FLT32 )SEC_HR;

    day     = tmTime->tm_mday;
    month   = tmTime->tm_mon + 1;
    year    = tmTime->tm_year + 1900;
    yearDay = tmTime->tm_yday + 1;

    dstFlag = ( tmTime->tm_isdst > ZERO ) ? 1 : 0;


// Verify the value yearDay.

    days = dayYear( day, month, year );

    if( days != yearDay )
    {
      sprintf( Mesg, "(lightcontrol) Values yearDay (%d) and days (%d) are not the same.\n", yearDay, days );
      info( Mesg, LOGFILE );
    }

    if( ( day == 31 ) && ( month == 12 ) )
    {
      nextDays = 1;
    }

    else
    {
      nextDays = days + 1;
    }


// Check for first time entry.

    if( firsTime )
    {
      getSunTimes( days, dstFlag, &riseTime, &setTime );


// Check if localTime is less than riseTime.

      if( localTime < ( riseTime - RiseOffset ) )
      {
        sleepHours = riseTime - localTime - RiseOffset;

	gpioValue = ON;
      }
      

// Check if localTime is greater than setTime.

      else if( localTime > ( setTime + SetOffset ) )
      {
        getSunTimes( nextDays, dstFlag, &riseTime, &setTime );

	sleepHours = 24.0 + riseTime - localTime - RiseOffset;

	gpioValue = ON;
      }


// Then localTime is greater than riseTime and less than setTime.

      else
      {
        sleepHours = setTime - localTime + SetOffset;

	gpioValue = OFF;
      }


      info( "(lightcontrol) Concluding First Time processing.\n", LOGFILE );

      firsTime = FALSE;
    }

    else
    {

// Check if localTime is in the morning.

      if( localTime < 12.0 )
      {
        getSunTimes( days, dstFlag, &riseTime, &setTime );

	sleepHours = setTime - localTime + SetOffset;

	if( sleepHours < FZERO )
	{
	  sprintf( Mesg, "(lightcontrol) Error, sleepHours is negative (%f).\n", sleepHours );
	  info( Mesg, LOGFILE );

	  sleepHours = 1.0;				// 1 hour
	}

	gpioValue = OFF;
      }

      
// Then localTime is in the afternoon/evening.

      else
      {
        getSunTimes( nextDays, dstFlag, &riseTime, &setTime );

	sleepHours = 24.0 + riseTime - localTime - RiseOffset;

	if( sleepHours < FZERO )
	{
	  sprintf( Mesg, "(lightcontrol) Error, sleepHours is negative (%f).\n", sleepHours );
	  info( Mesg, LOGFILE );

	  sleepHours = 1.0;				// 1 hour
	}

	gpioValue = ON;
      }
    }


    sprintf( Mesg, "(lightcontrol) localTime = %f, riseTime = %f, setTime = %f\n", localTime, riseTime, setTime );
    info( Mesg, LOGFILE );

    sleepSeconds = ( INT32 )( sleepHours * ( FLT32 )SEC_HR );

    sprintf( Mesg, "(lightcontrol) sleepHours = %f, sleepSeconds = %d, gpioValue = %s\n", sleepHours, sleepSeconds, onoff[gpioValue] );
    info( Mesg, LOGFILE );


// Set the gpio value.

    status = setGPIO( Gpio, gpioValue );

    if( status == ERROR )
    {
      sprintf( Mesg, "\n(lightcontrol) Error from setGPIO() for gpio #%d, gpioValue = %d.\n", Gpio, gpioValue );
      info( Mesg, LOGFILE );

      fclose( LogFilePtr );

      break;
    }


    info( "\n", LOGFILE );

    fclose( LogFilePtr );


// Sleep until next event.

    sleep( sleepSeconds );
  }


// Open the logfile in append mode.

  LogFilePtr = fopen( logPath, "a" );

  if( LogFilePtr == ( FILE * )NULL )
  {
    exit( EXIT_FAILURE );
  }


// Turn the gpio OFF.

  status = setGPIO( Gpio, OFF );

  if( status == ERROR )
  {
    sprintf( Mesg, "\n(lightcontrol) Error from setGPIO() for gpio #%d, off, at exit.\n", Gpio );
    info( Mesg, LOGFILE );
  }


// Close the gpio listed in the configuration file.

  sprintf( Mesg, "\n(lightcontrol) Calling closeGPIO() for gpio #%d.\n\n", Gpio );
  info( Mesg, LOGFILE );

  status = closeGPIO( Gpio );

  if( status )
  {
    sprintf( Mesg, "\n(lightcontrol) Error from closeGPIO() for gpio #%d.\n", Gpio );
    info( Mesg, LOGFILE );
  }


  info( "\n(lightcontrol) Program exiting.\n\n", LOGFILE );


  fclose( LogFilePtr );
}

